program Pms2hp48;  { convert a pm/ps graphic to HP-48SX format }

{ Copyright (c) 1990        Version 1        17 Mar 1990

  Clint Hastings - KC7XX
  408 East 2nd Avenue
  Salt Lake City, Utah 84103

  Freely redistributable, not public domain.  All rights reserved.
}

{$V-}      { to allow different size string passing }
(* {$R+}  *)    { enable range checking }

const MaxPicRecs = 64;   { per dynamically allocated memory block }

type  CPUreg = record  case boolean of
                 false: (ax,bx,cx,dx,bp,si,di,ds,es,flags : integer);
                 true:  (al,ah,bl,bh,cl,ch,dl,dh          : byte)
               end;

      PSpic = array[0..51,0..10] of byte;
      PMpic = record
                d1 : array[0..3] of byte; { dummy filler }
                p  : PSpic;
                d2 : byte
              END;
      PMpicsArray = array[0..MaxPicRecs] of PMpic;
      PSpicsArray = array[0..MaxPicRecs] of PSpic;
      PMpicsType = record case boolean of
            true  : (m : array[0..3] of ^PMpicsArray);
            false : (s : array[0..3] of ^PSpicsArray)
            END;

      CharSet = set of char;
      Format = (Caps, Norm, Auto);  { for GetString :
                                      Caps = all caps, don't need shift lock
                                      Norm = as entered
                                      Auto = first letter is automatic caps }
      String80 = string[80];
      String40 = string[40];
      String20 = string[20];
      String255 = string[255];
      Pname = array[1..16] of char;

const PicWidth   = 88;           { # of graphic bytes to send }
      LeftMargin = 9;
      Esc        = #27;
      UpArrow    = #1;
      DownArrow  = #3;
      USER       = 1;    { for ABORT }
      ERROR      = 2;
      PROGRAMEND = 0;
      NEXT       = -1;
      hx = 5;
      hy = 23;
      NumSet   : CharSet = ['0'..'9'];  { for GetString, GetInteger }
      FnameSet : CharSet = [' '..'}'];  { Chars valid for filenames }
      MaxNames = 300;
      FILEtf = 1;
      FILEpf = 2;
      FILEdf = 3;

var  reg : CPUreg;
     ch : char;
     PicMode, RAMmode, FileMode, PSmode, Inverse, debug, GotNames,
         DirMode, OverRide, ColorMode, VideoReset : boolean;
     FilesOpen : set of 1..8;
     filename : string80;
     MaxPic, g, n, Graphic, NumLabels, Col, Row, x,
         NumNames, NumDirNames, NumCols : integer;
     DirNames : array[1..70] of string40;
     Names : array[0..MaxNames] of Pname;
     convname : string[16];
     GraphicsMode : string[30];
     TestStr : string[20];
     DefaultGraphicDir, DefaultLabelDir : string40;
     OldVideoMode : byte;
     PMpics  : PMpicsType;
     df, tf  : text;
     nf, pf  : file;
     pic     : PSpic;

     TC      : char;  { holds last char used to terminate input }



(*  ConstStr returns a string length of N filled with character C *)
function ConstStr(C : Char; N : Integer) : String255;
var  S : string80;
begin
  if N < 0 then  N := 0;
  S[0] := Chr(N);
  FillChar(S[1],N,C);
  ConstStr := S
end;


procedure UpCaseStr(VAR s : string255);
var  i : integer;
begin
  for i := 1 to length(s) do  s[i] := UpCase(s[i])
end;


(* trims leading spaces from string (left side) *)
procedure TrimL(var s : string80);
begin
  while copy(s,1,1) = ' ' do delete(s,1,1)
end;

(* trims trailing spaces from string (right side) *)
procedure TrimR(var s : string80);
begin
  if length(s) > 0 then
      while copy(s,length(s),1) = ' ' do s[0] := pred(s[0])
end;

(* trims leading & trailing spaces from string *)
procedure Trim(var s : string80);
begin
  TrimL(s);
  TrimR(s)
 end;


function GetKB(var  scan, ascii : byte) : char;
begin
  reg.ax := 0;           { used to get alternate keys }
  intr($16,reg);
  ascii := lo(reg.ax);
  scan := hi(reg.ax);
  GetKB := char(ascii)
end;

function GetCh(OKset: CharSet): char;  { gets & echoes a char }  { getascii }
var  ch: char;                         { used by all input routines }
     scan, ascii : byte;
begin
  repeat
    ch := GetKB(scan, ascii);
    if ch = #0 then
      case scan of
        { 59..68 : ;      f1 to f10 }
        79 : ch := #10;  { end - same as ctrl-return }
        71 : ch := #4;   { home }
        72 : ch := UpArrow;   { up arrow }
        75 : ch := #2;   { left arrow }
        80 : ch := DownArrow;   { down arrow }
        77 : ch := #9    { right arrow - same as tab }
        END;
    if not(ch in OKset) then write(#7)
  until ch in OKset;

  if ch >= ' ' then  write(ch);      { eliminate these for no echo }
  if ch = #8 then  write(#8' '#8);
  TC := ch;
  GetCh := ch
end;

(* GetString
   Always starts from present cursor position
   OKset is the set of acceptable characters
   Accepts string passed if you just hit a termination char
   Zeroes field and starts over if you hit any other key
   BE SURE to initialize string s before you pass it to this routine
*)
procedure GetString(var s  : string80;
                   strlen  : integer;
                   OKset,
                   TermSet : CharSet;
                   sType   : Format );

var  ch : char;
     stemp: string80;
     i: integer;
     Upper : boolean;
begin
  ch:=' ';  stemp:='';  Upper := true;
  write(s,constStr('_',strlen-length(s)));
  write(constStr(^H,strlen));

  repeat
    if length(stemp)=0 then  ch:=getch(OKset + TermSet)    { 1st  char }
      else if length(stemp)=strlen then  ch:=getch([^H] + TermSet) { last char }
        else ch:=getch(OKset + [^H] + TermSet);            { middle chars }

    if not (ch in TermSet) AND (length(stemp)=0) then BEGIN  { blank unused default }
        write(constStr('_',strlen-1));
        write(constStr(^H ,strlen-1))
        END;

    if ch = ^H then  BEGIN
          delete(stemp,length(stemp),1);    { BackSpace }
          write('_',^H)
          END
      else if not(ch in TermSet) then
             BEGIN
               if (Upper and (sType = auto)) or (sType = Caps)
                   then ch := UpCase(ch);
               write(^H,ch);
               Upper := ch = ' ';
               stemp := stemp + ch
             END

  until (ch in TermSet);
        TC := ch;
  if length(stemp) > 0 then s:=stemp;
  write(constStr(^H,length(stemp)));
  write(s,ConstStr(' ',strlen-length(s)))
end;


procedure ClearHelp;
begin
  gotoxy(1,hy);
  write(ConstStr(' ',78));
  gotoxy(1,hy+1);
  write(ConstStr(' ',77))
end;


procedure ABORT(e : integer);
var  ch : char;
begin
  ClearHelp;
  gotoxy(hx,hy);
  if e = user then write('Aborting at user request. ')
    else if e = error then write('Aborting! ');
  {$I-}
  if FILEtf in FilesOpen then close(tf);
  if FILEdf in FilesOpen then close(df);
  if FILEpf in FilesOpen then close(pf);
  {$I+}
  if PicMode and VideoReset then BEGIN    { restore original video mode }
      reg.ah := 15;        { get current video mode }
      intr($10,reg);
      if OldVideoMode <> reg.al then BEGIN   { don't bother if same }
          gotoxy(hx,hy+1);
          write('Hit <RETURN> to exit...');
          readln(ch);
          reg.ah := 0;
          reg.al := OldVideoMode;
          intr($10,reg)
          END
      END;
  HALT
end;


(* Yes or No input routine - space or ^M accepts default value
   default value ( passed ) shows on screen *)
function YESNO(default: boolean): boolean;
var  ch: char;
begin
  if default then write('Y'+^H)
             else write('N'+^H);
  ch := getch(['y','n','Y','N',^M,' ',Esc]);
  if ch = Esc then ABORT(user);
  if (ch=^M) or (ch=' ') then  YESNO := default
                         else  YESNO := (ch='y') OR (ch='Y')
end;


(* GetInteger
   Input an integer between low and high boundaries.
   <Return> accepts the default shown.  *)
function GetInteger(low, high, default : integer;
                               TermSet : CharSet) : integer;
var  strg : string[20];
     len,i,temp : integer;
     IntSet : CharSet;
begin
  repeat
    IntSet := NumSet;
    str(high,strg);           { calculate longest possible input string }
    Trim(strg);
    len := length(strg);
    str(low,strg);
    Trim(strg);
    if len < length(strg) then len := length(strg);
    if low < 0 then IntSet := IntSet + ['-'];

    str(default,strg);
    trim(strg);
    while length(strg) < len do  insert('_',strg,1);
    GetString(Strg,len,IntSet,TermSet,Norm);    { go get it }
    Trim(strg);
    while copy(strg,1,1) = '_' do  delete(strg,1,1);
    val(STRG,temp,i);
    if i <> 0 then temp := 0;

    if (temp < Low) or (temp > High) then
        write(^G,constStr(^H,len));
  until (temp >= Low) and (temp <= High);
  GetInteger := temp;
  write(constStr(^H,len),temp:len)
end;

procedure ShowGraphic(n : integer);  { on the screen }
var  y, x, i : integer;
     screen1 : array[0..99,0..79] of byte ABSOLUTE $B800:0;
     screen2 : array[0..99,0..79] of byte ABSOLUTE $BA00:0;
begin
  for y := 0 to 51 do
    if odd(y) then move(pic[y,0], screen2[y div 2 + 42, 50], 11)
              else move(pic[y,0], screen1[y div 2 + 42, 50], 11);
  if GotNames then BEGIN
      gotoxy(53,19);
      write(Names[n])
      END
end;


procedure ReadInPics(var n : integer);
var  i, a : integer;
begin
   a := 0;
   n := 0;
   repeat
       New(PMpics.m[a]);
       BlockRead(pf, PMpics.m[a]^[0], MaxPicRecs, i);
       n := n + i;
       a := succ(a)
   until i < MaxPicRecs;
   close(pf);
end;

(* get graphic and stuff in variable Pic *)
procedure GetGraphic(n : integer);
var  TempPMpic : PMpic;
     x,y : integer;
begin
  if RAMmode then BEGIN
      if n = NEXT then BEGIN
          if Graphic < NumNames-1 then Graphic := succ(Graphic)
                                  else Graphic := 0;
          n := Graphic
          END;
      if PSmode then Pic := PMpics.s[n shr 6]^[n and $3F]
        else Pic := PMpics.m[n shr 6]^[n and $3F].p
      END
    else BEGIN
      if n > NEXT then  seek(pf,n)
        else BEGIN
          if Graphic < NumNames-1 then
              Graphic := succ(Graphic);
          n := Graphic
          END;
      if PSmode then BlockRead(pf,Pic,1)
        else BEGIN
          BlockRead(pf,TempPMpic,1);
          Pic := TempPMpic.p
          END
      END;
  if Inverse then
    for x := 0 to 51 do
      for y := 0 to 10 do
        Pic[x,y] := 255-Pic[x,y];

  convname := '';
  x := 1;
{  while (Names[n][x] != ' ') and (x < 9) do BEGIN
    convname := convname + Names[n,x];
    x := x + 1;
    END;}
  convname := 'grob'; (* copy(Names[n],1,8); *)
  convname := convname + '.g48';

  if PicMode then ShowGraphic(n)
    else if GotNames then BEGIN
        gotoxy(45,11);
        writeln(Names[n])
        END
end;


procedure WriteBackHex(b : byte);
var  i : array[0..1] of integer;
	 j : integer;
begin
  i[0] := b shr 4;
  i[1] := b AND $F;
  for j := 0 to 1 do
    case i[j] of
       0 : write(tf,'0');
       1 : write(tf,'8');
       2 : write(tf,'4');
       3 : write(tf,'C');
       4 : write(tf,'2');
       5 : write(tf,'A');
       6 : write(tf,'6');
       7 : write(tf,'E');
       8 : write(tf,'1');
       9 : write(tf,'9');
      10 : write(tf,'5');
      11 : write(tf,'D');
      12 : write(tf,'3');
      13 : write(tf,'B');
      14 : write(tf,'7');
      15 : write(tf,'F')
    END;  { case }
end;


procedure ConvertGraphic;
var   y, x : integer;
begin
  assign(tf,convname);
  rewrite(tf);
  write(tf,'' GROB 88 52 ');

  for y := 0 to 51 do
    for x := 0 to 10 do
      WriteBackHex(Pic[y,x]);

  writeln(tf,' ');
  {$I-}
  close(tf);
  {$I+}
end;


procedure InfoScreen;
begin
  gotoxy(3,1);
  writeln('Copyright (c) 1990 - Clint Hastings - Salt Lake City, Utah');
  gotoxy(5,3);
  write('************  P');
  if OverRide then
      if PSmode then write('S')
                   else write('M')
  else write('M / PS');
  writeln('  Graphic Convertor  ************'#10#10)
end;


procedure Help(s : string80);
var  oldX, oldY : integer;
begin
  oldX := whereX;
  oldY := whereY;
  gotoxy(hx,hy);
  write(s);
  gotoxy(oldX,oldY)
end;


procedure Parse(VAR line     : string40;
                    match    : string40;
                VAR variable : string40);
var  p, error : integer;
     nstr : string[5];
begin
  if copy(line,1,length(match)) = match then BEGIN
      variable := '';
      delete(line,1,length(match));   { delete identifier }
      p := pos(';',line);
      if p > 0 then  delete(line,p,1+length(line)-p);  { delete comments, if any }
      trim(line);
      while length(line) > 0 do BEGIN
          p := pos(',',line);
          if p > 0 then BEGIN
              nstr := copy(line,1,p-1);
              delete(line,1,p)
              END
            else BEGIN
              nstr := copy(line,1,length(line));
              line := ''
              END;
          val(nstr, p, error);
          if error = 0 then  variable := variable + chr(p)
          END
      END
end;


procedure ParseInt(VAR line     : string40;
                       match    : string40;
                   VAR variable : integer);
var  p, error : integer;
begin
  if copy(line,1,length(match)) = match then BEGIN
      delete(line,1,length(match));   { delete identifier }
      p := pos(';',line);
      if p > 0 then  delete(line,p,1+length(line)-p);  { delete comments, if any }
      trim(line);
      val(line,p,error);
      if error = 0 then  variable := p
      END
end;


procedure StrToInt(VAR line     : string40;
                       skip     : integer;  { number of chars to skip }
                   VAR variable : integer;
                   VAR OK       : boolean);
var  n, p, error : integer;
begin
  delete(line,1,skip);   { delete identifier }
  p := pos(';',line);
  if p > 0 then  delete(line,p,1+length(line)-p);  { delete comments, if any }
  trim(line);
  val(line,n,error);
  OK := error = 0;
  if OK then  variable := n
end;


procedure GetDirectory(VAR FileName   : string40;  { borrowed from some downloaded code }
                       VAR FilesExist : boolean);  { original source unknown }

type  buffertype   = record                        { NESTED procedures follow }
                        reserved : array[1..21] of byte;
                        attribute: byte;
                        time, date, FileSizeLo, FileSizeHi : integer;
                        name : string[13]
                      END;

VAR  i, LocationCode, ErrorReturn, OldX,OldY : integer;
     FullPath, template : string40;
     ch : char;
     error : byte;
     buffer : buffertype;


procedure SetDTA(VAR buff);
BEGIN
  with reg do BEGIN
      AX := $1A00;
      DS := seg(buff);
      DX := ofs(buff);
      MSDOS(reg)
    END
END;

procedure FindNext(VAR filename  : string40;
                   VAR Nexterror : integer);
VAR  carryflag, ci, di : integer;
     N : byte;
BEGIN
  with reg do BEGIN
      buffer.name := '             ';
      ah := $4F;
      MSDOS(reg);
      filename := '';
      carryflag := 1 and Flags;
      if (al = 2) or (al = 18) then carryflag := 1;
      If carryflag = 1 then  Nexterror := al
        else with buffer do BEGIN
            Nexterror := 0;
            for N := 0 to 12 do FileName := FileName + name[N];
            Trim(FileName);
            if FileName[length(FileName)] = #0 then
                Filename[0] := pred(Filename[0]);
            FileName := FullPath + FileName
            END
      END
END;

Procedure FindFirst (VAR filename  : string40;
                     VAR errorcode : integer );
VAR  carryflag, ci, di : integer;
      mask, N : byte;
BEGIN
    SetDTA(buffer);
    filename[ Length(filename) + 1] := chr(0);
    buffer.name := '             ';
    With reg do BEGIN
        ah := $4E;
        CX := 32;  {attribute}
        DS := Seg(filename);
        DX := Ofs(filename) + 1;
        MsDos(reg);
        { If there was an error set the error code and don't do
          anything else. }

        carryflag := 1 and Flags;
        if (al = 2) or (al = 18) then carryflag := 1;
        If carryflag = 1 then  errorcode := al
          else with buffer do BEGIN
            errorcode := 0;
            filename := '';
            for N := 0 to 12 do FileName := FileName + name[N];
            Trim(FileName);
            if FileName[length(FileName)] = #0 then
                Filename[0] := pred(Filename[0]);
            FileName := FullPath + FileName
            END
        END  {with}
END;

procedure GetAllFiles(VAR template    : string40;
                      VAR ErrorReturn : integer);
VAR  OneChar, ch, EscChoice : char;
     error, margin : integer;
BEGIN
  buffer.name := '             ';
  FindFirst(template, error);
  ErrorReturn := error;
  if error = 0 then BEGIN
    NumDirNames := succ(NumDirNames);
    DirNames[NumDirNames] := copy(template,1,40);
    repeat
      buffer.name := '             ';
      findNext(template,error);
      if error = 0 then BEGIN
        NumDirNames := succ(NumDirNames);
        DirNames[NumDirNames] := copy(template,1,40)
        END
    until error <> 0;
    if (error in [18,19]) then ErrorReturn := 0
  END
END;

function lastPos(C: char; S : string40) : byte;
VAR  temp, holder : byte;
BEGIN
  temp := 0;
  while pos(C,S) <> 0 do BEGIN
      temp := temp + pos(C,S);
      delete(S,1,pos(C,S))
      END;
  lastPos := temp
END;

BEGIN                   { GetDirectory }
  template := filename;
  FullPath := template;
  if lastPos('\',FullPath) <> 0 then
      FullPath := copy(FullPath,1,lastPos('\',FullPath))
    else if pos(':',FullPath) <> 0 then FullPath := copy(FullPath,1,2)
                                   else FullPath := '';
  GetAllFiles(template,ErrorReturn);
  FilesExist := ErrorReturn <> 1;
END;

procedure ReturnFileName(VAR filename : string40; FileType : integer);
label  ESCAPE;
var  OldFilename : string40;
     dn, oldX,oldY, startX, startY : integer;
     FilesExist : boolean;
begin
  startX := whereX;
  startY := whereY;
      REPEAT
        OldFileName := FileName;
        gotoxy(startX,startY);
        GetString(FileName,40,FnameSet,[UpArrow,DownArrow,#13,Esc],Caps);
        if TC = Esc then
            if FileType = 2 then GOTO ESCAPE
                            else ABORT(user);
        if Filename[length(FileName)] in [':','\'] then
            TC := DownArrow;
        if (TC = UpArrow) or (TC = DownArrow) then BEGIN
            if (OldFileName <> FileName) or (NumDirNames = 0) then BEGIN
                NumDirNames := 0;
                if (pos('*',filename) = 0) then  { no wild cards }
                  if FileType = 1 then BEGIN
                    if OverRide then
                      if PSmode then FileName := FileName + '*.DAT'
                                   else FileName := FileName + '*.SHP'
                    else FileName := FileName + '*.DAT'
                    END
                  else if FileType = 2 then BEGIN
                    FileName := FileName + '*.LAB'
                    END;

                GetDirectory(Filename,FilesExist);
                if FileType = 1 then
                  if not OverRide then BEGIN { in Auto mode, also check for PM files }
                    FileName := copy(FileName,1,length(Filename)-3) + 'SHP';
                    GetDirectory(FileName,FilesExist)
                    END;
                if NumDirNames = 0 then BEGIN
                    OldX := whereX;
                    OldY := whereY;
                    GotoXY(1,24);
                    WriteLn('No files found.');
                    gotoxy(oldX,oldY)
                    END;
                dn := 1;
                Filename := DirNames[dn]
                END;
            if (TC = DownArrow) and (dn > 1) then BEGIN
                dn := pred(dn);
                FileName := DirNames[dn]
                END
              else if (TC = UpArrow) and (dn < NumDirNames) then BEGIN
                dn := succ(dn);
                FileName := Dirnames[dn]
                END
            END;
        if TC = Esc then
            if FileType = 2 then GOTO ESCAPE
                            else ABORT(user)
      UNTIL TC = #13;
ESCAPE: ;   { want to enter text manually - forget about file }
end;

procedure Setup;
var  line, filename2 : string40;
     i, j : integer;
     ch : char;
     OK, EndOfString : boolean;
begin
  ClrScr;
  TC := ' ';
  Graphic := 0;
  if PicMode then BEGIN
      reg.ah := 15;   { save old video mode }
      intr($10,reg);
      OldVideoMode := reg.al;
      reg.ah := 0;
      reg.al := 6;  { set graphics mode -> 6 is 640x200 B&W }
      intr($10,reg)
      END
    else ClrScr;

  InfoScreen;

  gotoxy(5,9);
  write('Graphic file ? ');
  if DirMode then BEGIN
      Help('Enter a file directory or filename, then press RETURN.');
      gotoxy(hx,hy+1);
      write('Press UP or DOWN ARROW to scroll through names, RETURN to select.');
      FileName := DefaultGraphicDir;
      NumDirNames := 0;
      gotoxy(20,9);
      ReturnFileName(FileName,1)
      END
  else BEGIN
      Help('Enter a graphic filename and press Return.');
      if PSmode then FileName := 'B:GRLIB1.DAT'
                   else FileName := 'B:SLIB.SHP';
      GetString(FileName,40,FnameSet,[#13,Esc],Caps);
      if TC = Esc then ABORT(user)
      END;

  ClearHelp;
  if not OverRide then   { auto-select type of graphic file }
      PSmode := ( copy(FileName,length(FileName)-2,3) = 'DAT' );

  if PSmode then BEGIN
      assign(pf,FileName);
      {$I-}  reset(pf, sizeof(PSpic))  {$I+}
      END
    else BEGIN
      assign(pf,FileName);
      {$I-}  reset(pf, sizeof(PMpic))  {$I+}
      END;
  if IOresult <> 0 then BEGIN
      gotoxy(hx,hy);
      write(#7'Couldn''t find file "',Filename,'".');
      gotoxy(hx,hy+1);
      write('Please consult documentation.');
      ABORT(error)
      END;
  FilesOpen := FilesOpen + [FILEpf];
  NumNames := MaxNames;
  if RAMmode then  ReadInPics(NumNames);

  GotNames := false;
  filename2 := copy(filename,1,length(filename)-3);
  if PSmode then filename2 := filename2 + 'NAM'
               else filename2 := filename2 + 'SDR';
  assign(nf,fileName2);
  {$I-}  reset(nf, sizeof(Pname));  {$I+}
  if IOresult <> 0 then BEGIN
      gotoxy(hx,hy);
      write(#7'Couldn''t find file "',filename,'".');
      gotoxy(hx,hy+1);
      write('Graphic names unavailable.')
      END
    else BEGIN
      GotNames := true;
      i := NumNames;
      BlockRead(nf, Names[0], i, NumNames);
      {$I-} close(nf); {$I+}

(*      for i := 0 to NumNames-1 do BEGIN  { get rid of garbage in strings }
        EndOfString := false;
        for j := 1 to 16 do
          if not(Names[i][j] in [' '..#126]) or EndOfString then BEGIN
            EndOfString := true;
            Names[i][j] := ' '
            END
        END  *)
      END;

end;

procedure MainLoop;
var  n, m, oldX, oldY, nMax : integer;
     MoreLabels : boolean;
begin
  TC := ' ';
  REPEAT
      gotoxy(5,11);
      write('Graphic number to convert ? ');
            oldX := whereX;
            oldY := whereY;
            gotoxy(hx,hy);
            write('Enter a number between 0 and ', NumNames-1,'.');
            if DirMode then BEGIN
                gotoxy(hx,hy+1);
                write('Or press the UP or DOWN ARROW keys to scroll through ');
                if PicMode then write('graphics.')
                           else write('numbers.')
                END;
            gotoxy(oldX,oldY);
            GetGraphic(graphic);
            gotoxy(oldX,oldY);

          REPEAT
            Graphic := GetInteger(0,NumNames-1,Graphic,
                                  [#13,Esc,UpArrow,DownArrow]);
            if (TC = UpArrow) or (TC = DownArrow) then BEGIN
                if (TC = DownArrow) and (Graphic > 0) then
                    Graphic := pred(Graphic)
                  else if (TC = UpArrow) and (Graphic < NumNames-1) then
                    Graphic := succ(Graphic);
                GetGraphic(Graphic);
                gotoxy(oldX,oldY)
                END;
            if TC = Esc then ABORT(user);
          UNTIL TC = #13;

      GetGraphic(Graphic);
      ClearHelp;

      gotoxy(5,13);
      n := Graphic;
      ConvertGraphic;


      MoreLabels := false;
      gotoxy(5,20);
      write('Convert more graphics (y/n) ? ');
      Help('NO exits the program.');
      MoreLabels := YesNo(false);
      ClearHelp;
      writeln;

   UNTIL not MoreLabels
end;

procedure PrintHelp;
begin
  ClrScr;
  gotoxy(6,2);
  writeln('PM / PS Graphic Conversion');
  gotoxy(6,5);
  writeln('Command Line Option   (Default)');
  writeln('     ~~~~~~~~~~~~~~~~~~~~~~   ~~~~~~~~~');
  writeln('     -DG  Default graphic dir (B:)');
  writeln('     -DL  Default label dir   (B:)');
  writeln('     -I   Inverse video       (Normal)');
  writeln('     -ND  No Directory        (Dir Mode)');
  writeln('     -NP  No pictures         (Pictures)');
  writeln('     -NR  No RAM              (RAM Mode)');
  writeln('     -NV  No Video reset      (Reset)');
  writeln('     -PM  PM mode             (Automatic)');
  writeln('     -PS  PS mode             (Automatic)');
  HALT
end;


procedure ParseOption(VAR c : string80);
var  OK : boolean;
     ch : char;
     i, n : integer;
begin
  if c = '?' then PrintHelp;
  if c = '-PS' then BEGIN
      OverRide := true;
      PSmode := true
      END
  else if c = '-PM' then BEGIN
      OverRide := true;
      PSmode := false
      END;
  if c = '-NP' then PicMode := false
  else if c = '-ND' then DirMode := false
  else if c = '-NV' then VideoReset := false
  else if c = '-NR' then RAMmode := false
  else if c = '-I'  then Inverse := true
  else if c = '-DEBUG' then debug := true;
  if copy(c,1,3) = '-DL' then  DefaultLabelDir := copy(c,4,99)
  else if copy(c,1,3) = '-DG' then  DefaultGraphicDir := copy(c,4,99)
end;


procedure ParseCommandLine;
var  c : string80;
     p, i : integer;
begin
  DefaultGraphicDir := 'B:';
  DefaultLabelDir := 'B:';
  PSmode := false;
  Inverse := false;
  DirMode := true;
  VideoReset := true;
  PicMode := true;
  OverRide := false;
  RAMmode := true;
  FilesOpen := [];
  GotNames := false;

  assign(df,'CONFIG.PMS');
  {$I-}  reset(df);  {$I+}
  if IOresult = 0 then BEGIN         { Found configuration file }
      readln(df,c);
      trim(c);
      while copy(c,1,3) <> 'END' do BEGIN
          UpCaseStr(c);
          ParseOption(c);
          readln(df,c);
          trim(c)
          END;
      {$I-}  close(df)  {$I+}
      END;

  for p := 1 to ParamCount do BEGIN  { handle command line }
      c := ParamStr(p);
      UpCaseStr(c);
      ParseOption(c)
      END
end;


begin  { MAIN }
   ParseCommandLine;
   Setup;

   MainLoop;

   ABORT(PROGRAMEND)
end.
